//tabs=4
/*jsl:option explicit*/
	//
	//
	// This script reports the ending of evening astronomical twilight and
	// the beginning of dawn astronomical twilight.
	//
	// Typically, this will converge to 1 second within 2-3 loops.
	// Since the Sun ephem produces J2000 coordinates, we use local
	// MEAN sidereal time instead of APPARENT for the hour angle.
	// These approximations will get us typically to within 
	// 0.001 degree of the target sun elevation. PLENTY CLOSE! 
	//
	//  If the sun never reaches the given depression below the horizon,
	//  then the times shown are equal and represent the time of maximum depression,
	//  but don't meet the definition of the specified twilight condition.
	//
	function nextDuskDawn(h0, doDawn)
	{
		var DEGRAD = 0.0174532925;
		var KT = new ActiveXObject("Kepler.Ephemeris"); 		// Re-usable Earth ephem (reverse = Sun!)
		KT.BodyType = 0;						// This is a Planet
		KT.Number = 3;							// Specifically, it's the Earth
		var tvec = new ActiveXObject("NOVAS.PositionVector");
	    
		var LatRad = Telescope.SiteLatitude * DEGRAD;	  		// Latitude in radians (constant)
		var h0Rad = h0 * DEGRAD;					// Sun altitude, radians
		var JD = Util.SysJulianDate;					// Start with tomorrow's date/Time
		var JDPrev = JD;
		var failed = false;
		//
		// Iteration Loop
		//
		do
		{
			var KA = KT.GetPositionAndVelocity(JD).toArray();	// Get Earth from Sun
			tvec.x = -KA[0];					// Reverse cartesian vector for Sun from Earth
			tvec.y = -KA[1];
			tvec.z = -KA[2];
			var SunRA = tvec.RightAscension;			// J2000 coordinates of Sun from Earth
			var SunDec = tvec.Declination;
			var SunDecRad = SunDec * DEGRAD;
			var HA = (Math.sin(h0Rad)-(Math.sin(LatRad) * Math.sin(SunDecRad))) / (Math.cos(LatRad) * Math.cos(SunDecRad));
			if (HA > -1.0)
				HA = Math.acos(HA) / DEGRAD;			// HA in degrees
			else {
				HA = 180.0;
				failed = true;						// the real sun can't reach the required elevation below the horizon. 
				}
			if (doDawn) HA = -HA;					// this, for next dawn
			var LMST = Util.Julian_GMST(JD) + (Telescope.SiteLongitude / 15.0); // LOCAL MEAN Sidereal Time
			JD = JD + (((HA / 15.0) - Util.HourAngle12(SunRA, LMST)) / 24.0);
			if(Util.SysJulianDate > JD)
				JD = JD + 1;					// Add a day
			else if(Math.abs(JD - JDPrev) < 0.0000115741)   
				break;						// Convergence to ~1 second accuracy; breaks out of repeat loop
			JDPrev = JD;
		} while(true);
	    
		if (failed) Console.PrintLine (h0 + " degree twilight condition not reached.");

		KT = null;							// Explicitly release these
		tvec = null;
	    
		return new Date(Util.Julian_Date(JD));				// Local time of nearest dawn or dusk
	}

	// -----------
	// Main script
	// -----------
	//
function main()
{
	var sunset = -0.833333;							// upper limb at horizon, includes refraction correction
	var civil = -6.0;								// need artificial illumination to read outside
	var nautical = -12.0;							// navigation using sea horizon no longer possible
	var amateur = -15.0;							// dark enough for most astronomical observations
	var astronomical = -18.0;						// sky is completely dark
//
    Console.PrintLine("Dusk sunset is at : " + nextDuskDawn(sunset, false));
    Console.PrintLine("Dusk civil twilight ends at: " + nextDuskDawn(civil, false));
    Console.PrintLine("Dusk nautical twilight ends at: " + nextDuskDawn(nautical, false));
    Console.PrintLine("Dusk amateur twilight ends at: " + nextDuskDawn(amateur, false));
    Console.PrintLine("Dusk astronomical twilight ends at: " + nextDuskDawn(astronomical, false));
	Console.PrintLine(" ");
    Console.PrintLine("Dawn astronomical twilight begins at: " + nextDuskDawn(astronomical, true));
    Console.PrintLine("Dawn amateur twilight begins at: " + nextDuskDawn(amateur, true));
    Console.PrintLine("Dawn nautical twilight begins at: " + nextDuskDawn(nautical, true));
    Console.PrintLine("Dawn civil twilight begins at: " + nextDuskDawn(civil, true));
    Console.PrintLine("Dawn sunrise is at: " + nextDuskDawn(sunset, true));
 	}










































